@@ -144,7 +144,7 @@ static const struct var variables[] = {
 #define ERROR_SET    5 /* a --set problem */
 #define ERROR_MEM    6 /* out of memory */
 #define ERROR_URL    7 /* could not get a URL out of the set components */
-#define ERROR_TRIM   8 /* a --trim problem */
+#define ERROR_TRIM   8 /* a --qtrim problem */
 #define ERROR_BADURL 9 /* if --verify is set and the URL cannot parse */
 #define ERROR_GET   10 /* bad --get syntax */
 #define ERROR_ITER  11 /* bad --iterate syntax */
@@ -244,6 +244,7 @@ static void help(void)
     "      --keep-port                  - keep known default ports\n"
     "      --no-guess-scheme            - require scheme in URLs\n"
     "      --punycode                   - encode hostnames in punycode\n"
+    "      --qtrim [what]               - trim the query\n"
     "      --query-separator [letter]   - if something else than '&'\n"
     "      --quiet                      - Suppress (some) notes and comments\n"
     "      --redirect [URL]             - redirect to this\n"
@@ -251,9 +252,8 @@ static void help(void)
     "      --replace-append [data]      - appends a new query if not found\n"
     "  -s, --set [component]=[data]     - set component content\n"
     "      --sort-query                 - alpha-sort the query pairs\n"
-    "      --trim [component]=[what]    - trim component\n"
     "      --url [URL]                  - URL to work with\n"
-    "      --urlencode                  - URL encode components by default\n"
+    "      --urlencode                  - show components URL encoded\n"
     "  -v, --version                    - show version\n"
     "      --verify                     - return error on (first) bad URL\n"
     " URL COMPONENTS:\n"
@@ -652,6 +652,13 @@ static int getarg(struct option *o,
     *usedarg = gap;
   }
   else if(checkoptarg(o, "--trim", flag, arg)) {
+    if(strncmp(arg, "query=", 6))
+      errorf(o, ERROR_TRIM, "Unsupported trim component: %s", arg);
+
+    trimadd(o, &arg[6]);
+    *usedarg = gap;
+  }
+  else if(checkoptarg(o, "--qtrim", flag, arg)) {
     trimadd(o, arg);
     *usedarg = gap;
   }
@@ -1234,13 +1241,8 @@ static bool trim(struct option *o)
   bool query_is_modified = false;
   struct curl_slist *node;
   for(node = o->trim_list; node; node = node->next) {
-    char *ptr;
-    char *instr = node->data;
-    if(strncmp(instr, "query", 5))
-      /* for now we can only trim query components */
-      errorf(o, ERROR_TRIM, "Unsupported trim component: %s", instr);
-    ptr = strchr(instr, '=');
-    if(ptr && (ptr > instr)) {
+    char *ptr = node->data;
+    if(ptr) {
       /* 'ptr' should be a fixed string or a pattern ending with an
          asterisk */
       size_t inslen;
@@ -1248,7 +1250,6 @@ static bool trim(struct option *o)
       int i;
       char *temp = NULL;
 
-      ptr++; /* pass the = */
       inslen = strlen(ptr);
       if(inslen) {
         pattern = ptr[inslen - 1] == '*';
@@ -1340,7 +1341,7 @@ static char *encodequery(char *str, size_t len)
       *dupe++ = in;
     else {
       /* encode it */
-      const char hex[] = "0123456789ABCDEF";
+      const char hex[] = "0123456789abcdef";
       dupe[0]='%';
       dupe[1] = hex[in>>4];
       dupe[2] = hex[in & 0xf];
@@ -1661,9 +1662,6 @@ static bool replace(struct option *o)
     }
 
     if(!replaced && o->force_replace) {
-      trurl_warnf(o, "key '%.*s' not in url, appending to query",
-                  (int) (key.len),
-                  key.str);
       addqpair(key.str, strlen(key.str), o->jsonout);
       query_is_modified = true;
     }
@@ -1735,6 +1733,38 @@ static char *canonical_path(const char *path)
   return dupe;
 }
 
+static void normalize_part(struct option *o, CURLU *uh, CURLUPart part)
+{
+  char *ptr;
+  size_t ptrlen = 0;
+  (void)curl_url_get(uh, part, &ptr, 0);
+
+  if(ptr)
+    ptrlen = strlen(ptr);
+
+  if(ptrlen) {
+    int olen;
+    char *uptr;
+    /* First URL decode the component */
+    char *rawptr = curl_easy_unescape(NULL, ptr, (int)ptrlen, &olen);
+    if(!rawptr)
+      errorf(o, ERROR_MEM, "out of memory");
+
+    /* Then URL encode it again */
+    uptr = curl_easy_escape(NULL, rawptr, olen);
+    curl_free(rawptr);
+    if(!uptr)
+      errorf(o, ERROR_MEM, "out of memory");
+
+    if(strcmp(ptr, uptr))
+      /* changed, store the updated one */
+      (void)curl_url_set(uh, part, uptr, 0);
+    curl_free(uptr);
+  }
+  curl_free(ptr);
+}
+
+
 static void singleurl(struct option *o,
                       const char *url, /* might be NULL */
                       struct iterinfo *iinfo,
@@ -1852,7 +1882,7 @@ static void singleurl(struct option *o,
       char *cpath;
       bool path_is_modified = false;
       if(curl_url_get(uh, CURLUPART_PATH, &opath, 0))
-        errorf(o, ERROR_ITER, "out of memory");
+        errorf(o, ERROR_MEM, "out of memory");
 
       /* append path segments */
       for(p = o->append_path; p; p = p->next) {
@@ -1887,9 +1917,14 @@ static void singleurl(struct option *o,
       if(path_is_modified) {
         /* set the new path */
         if(curl_url_set(uh, CURLUPART_PATH, opath, 0))
-          errorf(o, ERROR_ITER, "out of memory");
+          errorf(o, ERROR_MEM, "out of memory");
       }
       curl_free(opath);
+
+      normalize_part(o, uh, CURLUPART_FRAGMENT);
+      normalize_part(o, uh, CURLUPART_USER);
+      normalize_part(o, uh, CURLUPART_PASSWORD);
+      normalize_part(o, uh, CURLUPART_OPTIONS);
     }
 
     query_is_modified |= extractqpairs(uh, o);
